@@ -6,10 +6,11 @@ from zope.interface import Interface
 from nevow import loaders, appserver
 from nevow.inevow import IRequest
 from nevow.util import resource_filename
+from allmydata import blacklist
 from allmydata.interfaces import ExistingChildError, NoSuchChildError, \
      FileTooLargeError, NotEnoughSharesError, NoSharesError, \
      EmptyPathnameComponentError, MustBeDeepImmutableError, \
-     MustBeReadonlyError, MustNotBeUnknownRWError
+     MustBeReadonlyError, MustNotBeUnknownRWError, SDMF_VERSION, MDMF_VERSION
 from allmydata.mutable.common import UnrecoverableFileError
 from allmydata.util import abbreviate
 from allmydata.util.encodingutil import to_str, quote_output
@@ -32,6 +33,49 @@ def parse_replace_arg(replace):
     else:
         return boolean_of_arg(replace)
 
+
+def get_format(req, default="CHK"):
+    arg = get_arg(req, "format", None)
+    if not arg:
+        if boolean_of_arg(get_arg(req, "mutable", "false")):
+            return "SDMF"
+        return default
+    if arg.upper() == "CHK":
+        return "CHK"
+    elif arg.upper() == "SDMF":
+        return "SDMF"
+    elif arg.upper() == "MDMF":
+        return "MDMF"
+    else:
+        raise WebError("Unknown format: %s, I know CHK, SDMF, MDMF" % arg,
+                       http.BAD_REQUEST)
+
+def get_mutable_type(file_format): # accepts result of get_format()
+    if file_format == "SDMF":
+        return SDMF_VERSION
+    elif file_format == "MDMF":
+        return MDMF_VERSION
+    else:
+        # this is also used to identify which formats are mutable. Use
+        #  if get_mutable_type(file_format) is not None:
+        #      do_mutable()
+        #  else:
+        #      do_immutable()
+        return None
+
+
+def parse_offset_arg(offset):
+    # XXX: This will raise a ValueError when invoked on something that
+    # is not an integer. Is that okay? Or do we want a better error
+    # message? Since this call is going to be used by programmers and
+    # their tools rather than users (through the wui), it is not
+    # inconsistent to return that, I guess.
+    if offset is not None:
+        offset = int(offset)
+
+    return offset
+
+
 def get_root(ctx_or_req):
     req = IRequest(ctx_or_req)
     # the addSlash=True gives us one extra (empty) segment
@@ -231,6 +275,9 @@ def humanize_failure(f):
              "The cap is being passed in a read slot (ro_uri), or was retrieved "
              "from a read slot as an unknown cap.") % quoted_name
         return (t, http.BAD_REQUEST)
+    if f.check(blacklist.FileProhibited):
+        t = "Access Prohibited: %s" % quote_output(f.value.reason, encoding="utf-8", quotemarks=False)
+        return (t, http.FORBIDDEN)
     if f.check(WebError):
         return (f.value.text, f.value.code)
     if f.check(FileTooLargeError):
